@@ -57,10 +57,7 @@ def tmp_local_or_remote_netcdf(request, tmpdir, hsds_up):
 
 @pytest.fixture(params=[True, False])
 def decode_vlen_strings(request):
-    if version.parse(h5py.__version__) >= version.parse("3.0.0"):
-        return dict(decode_vlen_strings=request.param)
-    else:
-        return {}
+    return dict(decode_vlen_strings=request.param)
 
 
 @pytest.fixture(params=[netCDF4, legacyapi])
@@ -489,8 +486,6 @@ def test_write_legacyapi_read_h5netcdf(tmp_local_netcdf, decode_vlen_strings):
 
 
 def test_fileobj(decode_vlen_strings):
-    if version.parse(h5py.__version__) < version.parse("2.9.0"):
-        pytest.skip("h5py > 2.9.0 required to test file-like objects")
     fileobj = tempfile.TemporaryFile()
     write_h5netcdf(fileobj)
     read_h5netcdf(fileobj, h5netcdf, decode_vlen_strings)
@@ -554,6 +549,24 @@ def test_attrs_api(tmp_local_or_remote_netcdf):
         assert sorted(ds["x"].attrs) == ["foo", "units"]
 
 
+def test_shape_is_tied_to_coordinate(tmp_local_or_remote_netcdf):
+    with h5netcdf.legacyapi.Dataset(
+        tmp_local_or_remote_netcdf,
+        "w",
+    ) as ds:
+        ds.createDimension("x", size=None)
+        ds.createVariable("xvar", int, ("x",))
+        ds["xvar"][:5] = np.arange(5)
+        assert ds["xvar"].shape == (5,)
+        ds.createVariable("yvar", int, ("x",))
+        ds["yvar"][:10] = np.arange(10)
+        assert ds["yvar"].shape == (10,)
+        # The shape of the xvar should change too
+        # this is in order to be in line with the behavior
+        # of netCDF4-c
+        assert ds["xvar"].shape == (10,)
+
+
 def test_optional_netcdf4_attrs(tmp_local_or_remote_netcdf):
     h5 = get_hdf5_module(tmp_local_or_remote_netcdf)
     with h5.File(tmp_local_or_remote_netcdf, "w") as f:
@@ -561,12 +574,8 @@ def test_optional_netcdf4_attrs(tmp_local_or_remote_netcdf):
         f.create_dataset("foo", data=foo_data)
         f.create_dataset("x", data=np.arange(5))
         f.create_dataset("y", data=np.arange(10))
-        if version.parse(h5py.__version__) < version.parse("2.10.0"):
-            f["foo"].dims.create_scale(f["x"])
-            f["foo"].dims.create_scale(f["y"])
-        else:
-            f["x"].make_scale()
-            f["y"].make_scale()
+        f["x"].make_scale()
+        f["y"].make_scale()
         f["foo"].dims[0].attach_scale(f["x"])
         f["foo"].dims[1].attach_scale(f["y"])
     with h5netcdf.File(tmp_local_or_remote_netcdf, "r") as ds:
@@ -589,15 +598,13 @@ def test_error_handling(tmp_local_or_remote_netcdf):
         ds.create_variable("x", ("x",), dtype=float)
         with raises(ValueError):
             ds.create_variable("x", ("x",), dtype=float)
+        with raises(ValueError):
+            ds.create_variable("y/", ("x",), dtype=float)
         ds.create_group("subgroup")
         with raises(ValueError):
             ds.create_group("subgroup")
 
 
-@pytest.mark.skipif(
-    version.parse(h5py.__version__) < version.parse("3.0.0"),
-    reason="not needed with h5py < 3.0",
-)
 def test_decode_string_error(tmp_local_or_remote_netcdf):
     write_h5netcdf(tmp_local_or_remote_netcdf)
     with pytest.raises(TypeError):
@@ -685,7 +692,7 @@ def test_fake_phony_dims(tmp_local_or_remote_netcdf):
 
 
 def check_invalid_netcdf4_mixed(var, i):
-    pdim = "phony_dim_{}".format(i)
+    pdim = f"phony_dim_{i}"
     assert var["foo1"].dimensions[0] == "y1"
     assert var["foo1"].dimensions[1] == "z1"
     assert var["foo1"].dimensions[2] == pdim
@@ -717,14 +724,9 @@ def test_invalid_netcdf4_mixed(tmp_local_or_remote_netcdf):
         for k, v in var2.items():
             f.create_dataset(k, data=np.arange(v))
 
-        if version.parse(h5py.__version__) < version.parse("2.10.0"):
-            f["foo2"].dims.create_scale(f["x1"])
-            f["foo2"].dims.create_scale(f["y1"])
-            f["foo2"].dims.create_scale(f["z1"])
-        else:
-            f["x1"].make_scale()
-            f["y1"].make_scale()
-            f["z1"].make_scale()
+        f["x1"].make_scale()
+        f["y1"].make_scale()
+        f["z1"].make_scale()
         f["foo2"].dims[0].attach_scale(f["x1"])
         f["foo2"].dims[1].attach_scale(f["y1"])
         f["foo2"].dims[2].attach_scale(f["z1"])
@@ -757,14 +759,9 @@ def test_invalid_netcdf_malformed_dimension_scales(tmp_local_or_remote_netcdf):
         f.create_dataset("y", data=np.arange(5))
         f.create_dataset("z", data=np.arange(5))
 
-        if version.parse(h5py.__version__) < version.parse("2.10.0"):
-            f["foo1"].dims.create_scale(f["x"])
-            f["foo1"].dims.create_scale(f["y"])
-            f["foo1"].dims.create_scale(f["z"])
-        else:
-            f["x"].make_scale()
-            f["y"].make_scale()
-            f["z"].make_scale()
+        f["x"].make_scale()
+        f["y"].make_scale()
+        f["z"].make_scale()
         f["foo1"].dims[0].attach_scale(f["x"])
 
     with raises(ValueError):
@@ -842,7 +839,7 @@ def test_failed_read_open_and_clean_delete(tmpdir):
     try:
         with h5netcdf.File(path, "r") as ds:
             assert ds
-    except IOError:
+    except OSError:
         pass
 
     # Look at garbage collection:
@@ -1024,14 +1021,25 @@ def test_creating_variables_with_unlimited_dimensions(tmp_local_or_remote_netcdf
         np.testing.assert_allclose(f.variables["dummy3"], np.zeros((3, 2)))
 
         # Writing to a variable with an unlimited dimension raises
-        with pytest.raises(TypeError) as e:
+        if tmp_local_or_remote_netcdf.startswith(remote_h5):
+            # We don't expect any errors. This is effectively a void context manager
+            expected_errors = memoryview(b"")
+        else:
+            expected_errors = pytest.raises(TypeError)
+        with expected_errors as e:
             f.variables["dummy3"][:] = np.ones((5, 2))
-        assert e.value.args[0] == "Can't broadcast (5, 2) -> (3, 2)"
+        if not tmp_local_or_remote_netcdf.startswith(remote_h5):
+            assert e.value.args[0] == "Can't broadcast (5, 2) -> (3, 2)"
         assert f.variables["dummy3"].shape == (3, 2)
         assert f.variables["dummy3"]._h5ds.maxshape == (None, 2)
         assert f["x"].shape == (3,)
         assert f.dimensions["x"].size == 3
-        np.testing.assert_allclose(f.variables["dummy3"], np.zeros((3, 2)))
+        if tmp_local_or_remote_netcdf.startswith(remote_h5):
+            # h5pyd writes the data, but does not expand the dimensions
+            np.testing.assert_allclose(f.variables["dummy3"], np.ones((3, 2)))
+        else:
+            # original data is kept for h5py
+            np.testing.assert_allclose(f.variables["dummy3"], np.zeros((3, 2)))
 
     # Close and read again to also test correct parsing of unlimited
     # dimensions.
@@ -1086,8 +1094,16 @@ def test_writing_to_an_unlimited_dimension(tmp_local_or_remote_netcdf):
         assert g.variables["dummy5"].shape == (3, 3)
 
         # broadcast writing
-        f.variables["dummy3"][...] = [[1, 2, 3]]
-        np.testing.assert_allclose(f.variables["dummy3"], [[1, 2, 3], [1, 2, 3]])
+        if tmp_local_or_remote_netcdf.startswith(remote_h5):
+            expected_errors = pytest.raises(OSError)
+        else:
+            # We don't expect any errors. This is effectively a void context manager
+            expected_errors = memoryview(b"")
+        with expected_errors as e:
+            f.variables["dummy3"][...] = [[1, 2, 3]]
+            np.testing.assert_allclose(f.variables["dummy3"], [[1, 2, 3], [1, 2, 3]])
+        if tmp_local_or_remote_netcdf.startswith(remote_h5):
+            assert "Got asyncio.IncompleteReadError" in e.value.args[0]
 
 
 def test_c_api_can_read_unlimited_dimensions(tmp_local_netcdf):
@@ -1354,10 +1370,7 @@ def create_netcdf_dimensions(ds, idx):
     collide[...] = np.arange(5 + idx)
     non_collide[...] = np.arange(5 + idx) + 10
     sample[0 : 2 + idx, : 2 + idx] = np.ones((2 + idx, 2 + idx))
-    if version.parse(h5py.__version__) >= version.parse("3.0.0"):
-        ship[0] = list("Skiff     ")
-    else:
-        ship[0] = string_to_char(np.array("Skiff     ", dtype="|S1"))
+    ship[0] = list("Skiff     ")
 
 
 def create_h5netcdf_dimensions(ds, idx):
@@ -1386,10 +1399,7 @@ def create_h5netcdf_dimensions(ds, idx):
     g.variables["collide"][...] = np.arange(5 + idx)
     g.variables["non_collide"][...] = np.arange(5 + idx) + 10
     g.variables["sample"][0 : 2 + idx, : 2 + idx] = np.ones((2 + idx, 2 + idx))
-    if version.parse(h5py.__version__) >= version.parse("3.0.0"):
-        g.variables["ship"][0] = list("Skiff     ")
-    else:
-        g.variables["ship"][0] = string_to_char(np.array("Skiff     ", dtype="|S1"))
+    g.variables["ship"][0] = list("Skiff     ")
 
 
 def check_netcdf_dimensions(tmp_netcdf, write_module, read_module):
@@ -1600,7 +1610,10 @@ def test_expanded_variables_netcdf4(tmp_local_netcdf, netcdf_write_module):
 
 
 # https://github.com/h5netcdf/h5netcdf/issues/136
-@pytest.mark.skip(reason="h5py bug with track_order prevents editing with netCDF4")
+@pytest.mark.skipif(
+    version.parse(h5py.__version__) < version.parse("3.7.0"),
+    reason="h5py<3.7.0 bug with track_order prevents editing with netCDF4",
+)
 def test_creation_with_h5netcdf_edit_with_netcdf4(tmp_local_netcdf):
     # In version 0.12.0, the wrong file creation attributes were used
     # making netcdf4 unable to open files created by h5netcdf
@@ -1936,7 +1949,7 @@ def test_array_attributes(tmp_local_netcdf):
         ds.attrs["unicode_fixed_0dim"] = np.array(unicode.encode("utf-8"), dtype=dt)
         ds.attrs["unicode_fixed_1dim"] = np.array([unicode.encode("utf-8")], dtype=dt)
         ds.attrs["unicode_fixed_arrary"] = np.array(
-            [unicode.encode("utf-8"), "foobár".encode("utf-8")], dtype=dt
+            [unicode.encode("utf-8"), "foobár".encode()], dtype=dt
         )
 
         dt = h5py.string_dtype("ascii", 10)
@@ -1966,25 +1979,18 @@ def test_array_attributes(tmp_local_netcdf):
         assert ds.attrs["unicode_list"] == unicode
 
         # bytes and strings are received as strings for h5py3
-        if version.parse(h5py.__version__) >= version.parse("3.0.0"):
-            ascii = "ascii"
-            foobar = "foobar"
-        # and bytes for h5py2
-        else:
-            ascii = b"ascii"
-            foobar = b"foobar"
+        ascii = "ascii"
+        foobar = "foobar"
         assert ds.attrs["ascii"] == "ascii"
         assert ds.attrs["ascii_0dim"] == ascii
         assert ds.attrs["ascii_1dim"] == ascii
         assert ds.attrs["ascii_array"] == [ascii, foobar]
-        # list is decoded for h5py2
         assert ds.attrs["ascii_list"] == "ascii"
 
         assert ds.attrs["bytes"] == ascii
         assert ds.attrs["bytes_0dim"] == ascii
         assert ds.attrs["bytes_1dim"] == ascii
         assert ds.attrs["bytes_array"] == [ascii, foobar]
-        # list is decoded for h5py2
         assert ds.attrs["bytes_list"] == "ascii"
 
         assert ds.attrs["unicode_fixed"] == unicode
@@ -2017,25 +2023,18 @@ def test_array_attributes(tmp_local_netcdf):
         assert ds.unicode_list == unicode
 
         # bytes and strings are received as strings for h5py3
-        if version.parse(h5py.__version__) >= version.parse("3.0.0"):
-            ascii = "ascii"
-            foobar = "foobar"
-        # and bytes for h5py2
-        else:
-            ascii = b"ascii"
-            foobar = b"foobar"
+        ascii = "ascii"
+        foobar = "foobar"
         assert ds.ascii == "ascii"
         assert ds.ascii_0dim == ascii
         assert ds.ascii_1dim == ascii
         assert ds.ascii_array == [ascii, foobar]
-        # list is decoded for h5py2
         assert ds.ascii_list == "ascii"
 
         assert ds.bytes == ascii
         assert ds.bytes_0dim == ascii
         assert ds.bytes_1dim == ascii
         assert ds.bytes_array == [ascii, foobar]
-        # list is decoded for h5py2
         assert ds.bytes_list == "ascii"
 
         assert ds.unicode_fixed == unicode
@@ -2078,9 +2077,7 @@ def test_array_attributes(tmp_local_netcdf):
         assert ds.bytes_0dim == ascii
         assert ds.bytes_1dim == ascii
         assert ds.bytes_array == [ascii, "foobar"]
-        # writing/reading lists is broken with h5py2/netCDF4
-        if version.parse(h5py.__version__) >= version.parse("3.0.0"):
-            assert ds.bytes_list == ascii
+        assert ds.bytes_list == ascii
 
         assert ds.unicode_fixed == unicode
         assert ds.unicode_fixed_0dim == unicode
